# ozi/filter.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
from __future__ import annotations

import hashlib
import re
from datetime import datetime
from datetime import timezone
from functools import lru_cache
from importlib.metadata import packages_distributions

import requests
from packaging.version import parse


def current_date(_format: str) -> str:
    """Filter to get the local date given format."""
    return datetime.now(tz=timezone.utc).date().strftime(_format)


@lru_cache
def underscorify(s: str) -> str:
    """Filter to replace non-alphanumerics with underscores."""
    return re.sub('[^0-9a-zA-Z]', '_', s)


@lru_cache
def wheel_repr(version: str) -> str:
    """Filter to transform versions of the form "X.Y" into "pyXY"."""
    major, minor = version.split('.')
    return f'py{major}{minor}'


@lru_cache
def sha256sum(version: str) -> str:  # pragma: no cover
    """Filter to transform OZI source version into a hash of the tarball.
    :param version: Version of OZI to get a hash for.
    :returns:
    """
    response = requests.get('https://pypi.org/pypi/OZI/json', timeout=30)
    match response.status_code:
        case 200:
            releases = response.json()['releases']
            latest_version = max(map(parse, releases.keys()))
            return str(
                [i for i in releases[latest_version] if i['filename'].endswith('.tar.gz')][
                    0
                ],
            )
        case _:
            checksum = hashlib.sha256()
            chunksize = 128 * 512
            response = requests.get(
                f'https://github.com/rjdbcm/OZI/archive/refs/tags/{version}.tar.gz',
                allow_redirects=True,
                stream=True,
                timeout=30,
            )
            for chunk in response.iter_content(chunksize):
                checksum.update(chunk)
            return checksum.hexdigest()


@lru_cache
def to_distribution(package: str) -> str | None:
    """Returns the first distributed module name for a package."""
    distributions = {v[0]: k for k, v in packages_distributions().items()}
    return distributions.get(package, None)



# ozi/experimental.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Assets for python packaging metadata."""
from __future__ import annotations

import gc
import sys
from contextlib import contextmanager
from contextlib import redirect_stderr
from contextlib import redirect_stdout
from contextlib import suppress
from runpy import run_module
from typing import Generator


@contextmanager
def redirect_argv(*args: str) -> Generator[None, None, None]:  # pragma: no cover
    argv = sys.argv[:]
    sys.argv = list(args)
    yield
    sys.argv = argv


@contextmanager
def nogc() -> Generator[None, None, None]:  # pragma: no cover
    gc.freeze()
    if gc.isenabled():
        gc.disable()
    yield
    gc.unfreeze()
    gc.enable()
    gc.collect()


def run_utility(name: str, *args: str) -> None:  # pragma: no cover
    with nogc():
        with redirect_argv(name, *args):
            with redirect_stdout(None):
                with redirect_stderr(None):
                    with suppress(SystemExit):
                        run_module(name)



# noqa: INP001
# ozi/scripts/sync_pkg_readme.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Check if the README.rst file is synchronized with PKG-INFO."""
import difflib
import email
import os
import pathlib
import sys

try:
    import toml  # type: ignore
except ImportError:
    import tomli as toml
# pylint: disable=consider-using-with
source = '/' / pathlib.Path(
    os.path.relpath(
        os.path.join('/', os.environ.get('MESON_SOURCE_ROOT', os.path.relpath('..'))),
        '/',
    ),
)
pkg_info_file = (source / 'PKG-INFO').open()
pkg_info = email.message_from_file(pkg_info_file).get_payload()
pkg_info_file.close()
readme_file = (source / 'README.rst').open()
readme = readme_file.read()
readme_file.close()
project_file = (source / 'pyproject.toml').open()
pyproject_toml = toml.loads(project_file.read())
project_file.close()
setuptools_scm = pyproject_toml.get('tool', {}).get('setuptools_scm', {})
write_to_template = email.message_from_string(
    setuptools_scm.get('write_to_template', ''),
).get_payload()
DIFF1 = ''.join(
    difflib.context_diff(
        pkg_info,
        readme,
        tofile='PKG-INFO',
        fromfile='README.rst',
        lineterm='',
    ),
)
DIFF2 = ''.join(
    difflib.context_diff(
        readme,
        write_to_template,
        tofile='pyproject.toml',
        fromfile='README.rst',
        lineterm='',
    ),
)
DIFF3 = ''.join(
    difflib.context_diff(
        write_to_template,
        pkg_info,
        tofile='pyproject.toml',
        fromfile='PKG-INFO',
        lineterm='',
    ),
)
if DIFF1 != '':
    print(DIFF1)
    sys.exit(len(DIFF1))
elif DIFF2 != '':
    print(DIFF2)
    sys.exit(len(DIFF2))
elif DIFF3 != '':
    print(DIFF3)
    sys.exit(len(DIFF3))
